import * as IntercomTypes from "./types";
import { regionAPIs } from "./constants";
import { init, ref } from "./instance-manager";

const callIntercomMethod = (method: string, ...args: any[]) => {
  // window is undefined on server-side and this breaks our widget
  if (typeof window !== undefined && window.Intercom) {
    window.Intercom(method, ...args);
  }

  else {
    console.warn("Please ensure Intercom is setup and running on client-side!");
  }
};

export const Intercom = (props: IntercomTypes.InitType) => {
  if (typeof props !== "object") {
    console.warn("Intercom initialiser called with invalid parameters.");
    return;
  }
  const { region = "us", ...args } = props;
  if (typeof window !== "undefined" && !ref) {
    window.intercomSettings = {
      ...args,
      api_base: regionAPIs.get(region),
    };
    init();
  }
};

// Public functions that can be called from outside the module
export default Intercom;

export const boot = (arg: IntercomTypes.IntercomSettings) => callIntercomMethod("boot", arg);
export const shutdown = () => callIntercomMethod("shutdown");
export const update = (arg: IntercomTypes.UserArgs) => callIntercomMethod("update", arg);
export const hide = () => callIntercomMethod("hide");
export const show = () => callIntercomMethod("show");
export const showSpace = (spaceName: string) => callIntercomMethod("showSpace", spaceName);
export const showMessages = () => callIntercomMethod("showMessages");
export const showNewMessage = (prePopulatedContent: string) => callIntercomMethod("showNewMessage", prePopulatedContent);
export const onHide = (callback: Function) => callIntercomMethod("onHide", callback);
export const onShow = (callback: Function) => callIntercomMethod("onShow", callback);
export const onUnreadCountChange = (callback: Function) => callIntercomMethod("onUnreadCountChange", callback);
export const trackEvent = (...args: any) => callIntercomMethod("trackEvent", ...args);
export const getVisitorId = () => callIntercomMethod("getVisitorId");
export const startTour = (tourId: string) => callIntercomMethod("startTour", tourId);
export const showArticle = (articleId: string) => callIntercomMethod("showArticle", articleId);
export const showNews = (newsItemId: string) => callIntercomMethod("showNews", newsItemId);
export const startSurvey = (surveyId: string) => callIntercomMethod("startSurvey", surveyId);
export const startChecklist = (checklistId: string) => callIntercomMethod("startChecklist", checklistId);
export const showTicket = (ticketId: string) => callIntercomMethod("showTicket", ticketId);
export const showConversation = (conversationId: string) => callIntercomMethod("showConversation", conversationId);
export const onUserEmailSupplied = (callback: Function) => callIntercomMethod("onUserEmailSupplied", callback);